# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-28 19:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0019_auto_20161121_0140'),
    ]

    operations = [
        migrations.AddField(
            model_name='work',
            name='can_be_done_remotely',
            field=models.BooleanField(default=False, verbose_name='This job can be done remotely'),
        ),
    ]
