# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-21 01:40
from __future__ import unicode_literals

from django.db import migrations, models
from django.utils import timezone


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0018_auto_20161118_1214'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jobdate',
            name='end_date',
            field=models.DateTimeField(default=timezone.now(), verbose_name='End date'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='jobdate',
            name='start_date',
            field=models.DateTimeField(default=timezone.now(), verbose_name='Start date'),
            preserve_default=False,
        ),
    ]
