# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-28 19:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_organizations', '0003_organization_type'),
        ('ovp_projects', '0007_project_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='organization',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to='ovp_organizations.Organization'),
            preserve_default=False,
        ),
    ]
